/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-07-12 15:31:10.
 * *************************************************
 */

package cn.com.qmth.importpaper;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.charset.Charset;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

import org.apache.commons.io.IOUtils;
import org.apache.log4j.Logger;


public class FileUtil {
    private static Logger log = Logger.getLogger(FileUtil.class);


    /**
     * 解压文件
     *
     * @param targetDir 解压目录
     * @param zipFile   待解压的ZIP文件
     */
    public static List<File> unZip(File targetDir, File zipFile) {
        if (targetDir == null) {
            log.error("解压目录不能为空！");
            return null;
        }

        if (zipFile == null) {
            log.error("待解压的文件不能为空！");
            return null;
        }

        if (!zipFile.exists()) {
            log.error("待解压的文件不存在！" + zipFile.getAbsolutePath());
            return null;
        }

        String zipName = zipFile.getName().toLowerCase();
        if (zipFile.isDirectory() || zipName.indexOf(".zip") < 0) {
            log.error("待解压的文件格式错误！");
            return null;
        }

        if (!targetDir.exists()) {
            targetDir.mkdir();
        }

        log.info("UnZip targetDir:" + targetDir.getAbsolutePath());
        List<File> result = new LinkedList<>();

        try (ZipFile zip = new ZipFile(zipFile, Charset.forName("UTF-8"));) {

            @SuppressWarnings("rawtypes")
			Enumeration entries = zip.entries();
            while (entries.hasMoreElements()) {
                ZipEntry entry = (ZipEntry) entries.nextElement();

                //Linux中需要替换掉路径的反斜杠
                String entryName = (File.separator + entry.getName()).replaceAll("\\\\", "/");
                log.info("UnZip:" + entryName);

                String filePath = targetDir.getAbsolutePath() + entryName;
                File target = new File(filePath);
                if (entry.isDirectory()) {
                    target.mkdirs();
                } else {
                    File dir = target.getParentFile();
                    if (!dir.exists()) {
                        dir.mkdirs();
                    }

                    try (OutputStream os = new FileOutputStream(target);
                         InputStream is = zip.getInputStream(entry);) {
                        IOUtils.copy(is, os);
                        os.flush();
                    } catch (IOException e) {
                        log.error(e.getMessage(), e);
                    }
                    result.add(target);
                }
            }

        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }

        return result;
    }

    /**
     * 文件压缩
     *
     * @param target  目录或文件
     * @param zipFile 压缩后的ZIP文件
     */
    public static boolean doZip(File target, File zipFile) {
        if (target == null || !target.exists()) {
            log.error("目录或文件不能为空！");
            return false;
        }

        if (zipFile == null) {
            log.error("待压缩的文件不能为空！");
            return false;
        }

        try (
                OutputStream outStream = new FileOutputStream(zipFile);
                ZipOutputStream zipOutStream = new ZipOutputStream(outStream, Charset.forName("UTF-8"));
        ) {
            if (!zipFile.exists()) {
                boolean ok = zipFile.createNewFile();
                if (!ok) {
                    log.error("压缩的文件创建失败！");
                    return false;
                }
            }

            if (target.isDirectory()) {
                File[] files = target.listFiles();
                if (files.length == 0) {
                    log.error("文件夹内未找到任何文件！");
                    return false;
                }

                for (File file : files) {
                    doZip(zipOutStream, file, null);
                }
            } else {
                doZip(zipOutStream, target, null);
            }
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }

        return true;
    }

    private static void doZip(ZipOutputStream zipOutStream, File target, String parentDir) throws IOException {
        //log.info("Zip:" + parentDir);
        if (parentDir == null) {
            parentDir = "";
        }

        if (!"".equals(parentDir) && !parentDir.endsWith(File.separator)) {
            parentDir += File.separator;
        }

        if (target.isDirectory()) {
            File[] files = target.listFiles();
            if (files.length > 0) {
                for (File file : files) {
                    doZip(zipOutStream, file, parentDir + target.getName());
                }
            } else {
                zipOutStream.putNextEntry(new ZipEntry(parentDir + target.getName()));
                zipOutStream.closeEntry();
            }
        } else {
            try (InputStream is = new FileInputStream(target);) {
                zipOutStream.putNextEntry(new ZipEntry(parentDir + target.getName()));
                int len;
                byte[] bytes = new byte[1024];
                while ((len = is.read(bytes)) > 0) {
                    zipOutStream.write(bytes, 0, len);
                }
            } catch (IOException e) {
                log.error(e.getMessage(), e);
            }
            zipOutStream.closeEntry();
        }
    }
    public static boolean doZip(List<File> source, File zipFile) {
        if (source == null || source.size()==0) {
            log.error("待压缩的文件不能为空！");
            return false;
        }

        if (zipFile == null) {
            log.error("压缩后的文件不能为空！");
            return false;
        }

        try (
                OutputStream outStream = new FileOutputStream(zipFile);
                ZipOutputStream zipOutStream = new ZipOutputStream(outStream, Charset.forName("UTF-8"));
        ) {
            if (!zipFile.exists()) {
                boolean ok = zipFile.createNewFile();
                if (!ok) {
                    log.error("压缩的文件创建失败！");
                    return false;
                }
            }
            for (File file : source) {
                doZip(zipOutStream, file, null);
            }
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }

        return true;
    }
}